// Bomb logic

#include "Hitters.as";
#include "BombCommon.as";
#include "ShieldCommon.as";

const s32 bomb_fuse = 120;

void onInit( CBlob@ this )
{	    
    this.set_u16("explosive_parent",0);
	this.getShape().getConsts().net_threshold_multiplier = 2.0f;
	SetupBomb( this, bomb_fuse, 64.0f, 3.0f, 24.0f, 0.4f, true );
    //
	this.Tag("activated"); // make it lit already and throwable
}

//start ugly bomb logic :)

void set_delay( CBlob@ this, string field, s32 delay)
{
    this.set_s32(field,getGameTime()+delay);
}

void onTick( CBlob@ this )
{
    //set parent from attached blob

    if (getNet().isServer())
    {
        CBlob@ parent = this.getAttachments().getAttachedBlob("PICKUP", 0);

        if (parent !is null)
        {
            u16 oldParentID = this.get_u16("explosive_parent");
            u16 newParentID = parent.getNetworkID();

            if (oldParentID != newParentID)
            {
                this.set_u16("explosive_parent", newParentID);
                this.SetDamageOwnerPlayer( parent.getPlayer() );
                //this.Sync("explosive_parent", true); we dont need this synced
            }
        }
    }

}

//sprite update

void onTick( CSprite@ this )
{
    CBlob@ blob = this.getBlob();
    Vec2f vel = blob.getVelocity();
    
    s32 timer = blob.get_s32("bomb_timer") - getGameTime();

    if (timer < 0) {
        return;
    }

    if (timer > 30)
    {
        this.SetAnimation("default");
        this.animation.frame = this.animation.getFramesCount() * (1.0f - ((timer-30) / 220.0f));
    }
    else
    {
        this.SetAnimation("shes_gonna_blow");
        this.animation.frame = this.animation.getFramesCount() * (1.0f - (timer / 30.0f));

        if (timer < 15 && timer > 0)
        {
            f32 invTimerScale = (1.0f-(timer / 15.0f));
            Vec2f scaleVec = Vec2f(1,1)*(1.0f + 0.07f*invTimerScale*invTimerScale );
            this.ScaleBy( scaleVec );
        }
    }	 		  
}

f32 onHit( CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitterBlob, u8 customData)
{
	const u8 hitter = this.get_u8("custom_hitter");
	if ( hitterBlob.getName() == ("arrow") )
	{
		CPlayer@ hitterPlayer = hitterBlob.getDamageOwnerPlayer();
		this.server_setTeamNum(hitterBlob.getTeamNum());	
		if (hitterPlayer !is null) 
		{
			u16 hpID = hitterPlayer.getNetworkID();
			this.set_u16("explosive_parent", hpID);
			this.SetDamageOwnerPlayer(hitterPlayer);
			if (hitter==Hitters::water)
			{
				this.set_s32("bomb_timer", 0);
				return damage;
			}
		}	

	}
	else if ( hitterBlob.getName() == ("bomb") && hitterBlob.getTeamNum() != this.getTeamNum()  )
	{		 
		this.set_s32("bomb_timer", 0);
		return damage; 
	}
	else if (hitterBlob.getName() == ("waterbomb"))
	{
		this.set_s32("bomb_timer", 0);
		return damage; 
	}

return 0.0f;
}

void onDie( CBlob@ this )
{
    this.getSprite().SetEmitSoundPaused( true );
}

bool doesCollideWithBlob( CBlob@ this, CBlob@ blob )
{
	//special logic colliding with players
	if (blob.hasTag("player"))
    {
		const u8 hitter = this.get_u8("custom_hitter");
		
		//all water bombs collide with enemies
		if(hitter == Hitters::water)
			return blob.getTeamNum() != this.getTeamNum();
		
		//collide with shielded enemies
		return (blob.getTeamNum() != this.getTeamNum() && blob.hasTag("shielded"));
    }
    
    return true;
}



void onCollision( CBlob@ this, CBlob@ blob, bool solid )
{
    if (!solid) {
        return;
    }

    const f32 vellen = this.getOldVelocity().Length();	  
	const u8 hitter = this.get_u8("custom_hitter");
    if (vellen > 1.7f)
    {
        Sound::Play( !isExplosionHitter(hitter) ? "/WaterBubble" :
                     "/BombBounce.ogg", this.getPosition(), Maths::Min( vellen/8.0f, 1.1f ) );
    }

	if (!isExplosionHitter(hitter))
	{
		Boom( this );
        if(!this.hasTag("_hit_water") && blob !is null) //smack that mofo
        {
            this.Tag("_hit_water");
            Vec2f pos = this.getPosition();
            blob.Tag("force_knock");
        }
	}
}

